/*
 * test.c
 *
 *  Created on: Sep 27, 2010
 *      Author: christian
 */

#include "dsm-api.h"
#include "dsm-api-const.h"
#include <digitalSTROM/ds.h>
#include <unistd.h>

void BusChange_callback(void *arg, dsid_t *id, int flag)
{
    int i;
    for (i = 0; i < 12; ++i)
    {
        printf("%02x ", id->id[i]);
    }
    if (flag)
    {
        printf("left bus\n");
    }
    else
    {
        printf("joined bus\n");
    }
}

void BusState_callback(void *arg, bus_state_t state)
{
    switch (state)
    {
    case DS485_ISOLATED:
        printf("STATE: ISOLATED\n");
        break;
    case DS485_CONNECTED:
        printf("STATE: CONNECTED\n");
        break;
    case DS485_ACTIVE:
        printf("STATE: ACTIVE\n");
        break;
    case DS485_JOIN:
        printf("STATE: JOIN\n");
        break;
    default:
        printf("STATE: *UNKNOWN*\n");
        break;
    }
}

void EventDeviceAccessibility_on(uint8_t error_code, void *arg, dsid_t sourceId, dsid_t destinationId, uint16_t DeviceId, uint16_t ZoneId, uint32_t DeviceDSID)
{
    int i;
    printf("Device 0x%08x (DeviceId: %d in Zone: %d) became active on dSM", DeviceDSID, DeviceId, ZoneId);
    for (i = 0; i < 12; i++)
    {
        printf(" %02x", sourceId.id[i]);
    }
    printf("\n");
}

void EventDeviceAccessibility_off(uint8_t error_code, void *arg, dsid_t sourceId, dsid_t destinationId, uint16_t DeviceId, uint16_t ZoneId, uint32_t DeviceDSID)
{
    int i;
    printf("Device 0x%08x (DeviceId: %d in Zone: %d) became inactive on dSM", DeviceDSID, DeviceId, ZoneId);
    for (i = 0; i < 12; i++)
    {
        printf(" %02x", sourceId.id[i]);
    }
    printf("\n");
}

void ZoneCount_callback(uint8_t error_code, void *arg, dsid_t sourceId, dsid_t destinationId, uint8_t ZoneCount)
{
    int i;
    printf("dSM:");
    for (i = 0; i < 12; i++)
    {
        printf(" %02x", sourceId.id[i]);
    }
    printf(" has ZoneCount: %d (result: %d)\n", ZoneCount, error_code);
}

void EventDeviceStatistics(uint8_t error_code, void *arg, dsid_t sourceId, dsid_t destinationId, uint16_t DeviceIndex, uint8_t Offset, uint16_t Value)
{
    printf("Device: %d, Offset %d: 0x%02x\n", DeviceIndex, Offset, Value);
}

void ZoneGroupActionRequest_action_call_scene_request(uint8_t error_code, void *arg, dsid_t sourceId, dsid_t destinationId, uint16_t ZoneId, uint8_t GroupId, uint8_t SceneId)
{
    int i;
    printf("Call scene %d in zone %d and group %d from dSM: ", SceneId, ZoneId, GroupId);
    for (i = 0; i < 12; i++)
    {
        printf(" %02x", sourceId.id[i]);
    }
    printf("\n");
}


int main(int argc, char* argv[])
{
    int result;
    const char connection[] = "tcp://192.168.2.124:8442";
    dsid_t dsid = { 0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11 };
    dsid_t broadcast;
    SetBroadcastId(broadcast);

    DsmApiHandle_t handle = DsmApiInitialize();

    result = DsmApiOpen(handle, connection, 0);

    DsmApiRegisterBusStateCallback(handle, BusState_callback, NULL);

    DsmApiRegisterBusChangeCallback(handle, BusChange_callback, NULL);

    DsmApiRegisterCallback(handle, DS485_CONTAINER_EVENT, EVENT_DEVICE_ACCESSIBILITY, EVENT_DEVICE_ACCESSIBILITY_ON, &EventDeviceAccessibility_on, NULL);
    DsmApiRegisterCallback(handle, DS485_CONTAINER_EVENT, EVENT_DEVICE_ACCESSIBILITY, EVENT_DEVICE_ACCESSIBILITY_OFF, &EventDeviceAccessibility_off, NULL);

    DsmApiRegisterCallback(handle, DS485_CONTAINER_RESPONSE, ZONE_COUNT, 0, &ZoneCount_callback, NULL);
    DsmApiRegisterCallback(handle, DS485_CONTAINER_EVENT, EVENT_DEVICE_STATISTICS, 0, &EventDeviceStatistics, NULL);

    DsmApiRegisterCallback(handle, DS485_CONTAINER_REQUEST, ZONE_GROUP_ACTION_REQUEST, ZONE_GROUP_ACTION_REQUEST_ACTION_CALL_SCENE, &ZoneGroupActionRequest_action_call_scene_request, NULL);

    DeviceStatisticsRequest_get(handle, dsid, 65, 0xff);

    sleep(10);

    ZoneCount(handle, broadcast, NULL);

    sleep(3600);
}
